using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using MLEM.Textures;
using Newtonsoft.Json;
using static TouchyTickets.Attractions.AttractionFlags;

namespace TouchyTickets.Attractions {
    [JsonConverter(typeof(Converter))]
    public class AttractionModifier {

        public static readonly Dictionary<string, AttractionModifier> Modifiers = new Dictionary<string, AttractionModifier>();

        static AttractionModifier() {
            Register(new AttractionModifier("Lubricant", 200, Cars | FastCars, 1.02F, Ui.Texture[0, 4]));
            Register(new AttractionModifier("LouderMusic", 500, Relaxed, 1.03F, Ui.Texture[2, 4]));
            Register(new AttractionModifier("SmallAds", 800, Small, 1.5F, Ui.Texture[5, 4]));
            Register(new AttractionModifier("LongerQueue", 1000, All, 1.06F, Ui.Texture[1, 4]));
            Register(new AttractionModifier("Bouncer", 1500, Walking, 1.2F, Ui.Texture[3, 4]));
            Register(new AttractionModifier("OnRideCameras", 2500, FastCars, 1.1F, Ui.Texture[4, 4]));
        }

        public readonly string Name;
        public readonly long InitialPrice;
        public readonly TextureRegion Texture;
        public readonly float Multiplier;
        private readonly AttractionFlags affectedFlags;

        public AttractionModifier(string name, long initialPrice, AttractionFlags affectedFlags, float multiplier, TextureRegion texture) {
            this.Name = name;
            this.InitialPrice = initialPrice;
            this.affectedFlags = affectedFlags;
            this.Texture = texture;
            this.Multiplier = multiplier;
        }

        public bool IsAffected(Attraction attraction) {
            return (attraction.Type.Flags & this.affectedFlags) != 0;
        }

        private static AttractionModifier Register(AttractionModifier type) {
            Modifiers.Add(type.Name, type);
            return type;
        }

        public class Converter : JsonConverter<AttractionModifier> {

            public override void WriteJson(JsonWriter writer, AttractionModifier value, JsonSerializer serializer) {
                if (value != null)
                    writer.WriteValue(value.Name);
            }

            public override AttractionModifier ReadJson(JsonReader reader, Type objectType, AttractionModifier existingValue, bool hasExistingValue, JsonSerializer serializer) {
                return reader.Value != null ? Modifiers[reader.Value.ToString()] : null;
            }

        }

    }

    [DataContract]
    public class ActiveModifier {

        [DataMember]
        public readonly AttractionModifier Modifier;
        [DataMember]
        public int Amount;

        public ActiveModifier(AttractionModifier modifier, int amount) {
            this.Modifier = modifier;
            this.Amount = amount;
        }

    }
}